<?php

namespace FiloBlu\Storelocator\Observer;

use Exception;
use Magento\Framework\App\Area;
use Magento\Framework\App\State;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Mail\Template\TransportBuilder;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Magento\Framework\Translate\Inline\StateInterface;
use Magento\Payment\Helper\Data;
use Magento\Quote\Model\QuoteFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order\Address\Renderer;
use Magento\Store\Model\StoreManagerInterface;
use Psr\Log\LoggerInterface;

/**
 *
 */
class OrderSave implements ObserverInterface
{
    /**
     * @var \FiloBlu\Storelocator\Helper\Data
     */
    protected $helper;

    /**
     * @var \Magento\Quote\Model\QuoteFactory
     */
    protected $quoteFactory;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $transportBuilder;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $inlineTranslation;

    /**
     * @var \Magento\Payment\Helper\Data
     */
    protected $paymentHelper;

    /**
     * @var \Magento\Sales\Model\Order\Address\Renderer
     */
    protected $addressRenderer;
    /**
     * @var State
     */
    protected $appState;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    private $date;

    /**
     * @param \FiloBlu\Storelocator\Helper\Data $helper
     * @param \Magento\Quote\Model\QuoteFactory $quoteFactory
     * @param \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Payment\Helper\Data $paymentHelper
     * @param \Magento\Sales\Model\Order\Address\Renderer $addressRenderer
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \FiloBlu\Storelocator\Helper\Data $helper,
        QuoteFactory $quoteFactory,
        OrderRepositoryInterface $orderRepository,
        TransportBuilder $transportBuilder,
        StoreManagerInterface $storeManager,
        StateInterface $inlineTranslation,
        Data $paymentHelper,
        Renderer $addressRenderer,
        DateTime $date,
        LoggerInterface $logger,
        State $appState

    ) {
        $this->helper = $helper;
        $this->quoteFactory = $quoteFactory;
        $this->orderRepository = $orderRepository;
        $this->transportBuilder = $transportBuilder;
        $this->storeManager = $storeManager;
        $this->inlineTranslation = $inlineTranslation;
        $this->paymentHelper = $paymentHelper;
        $this->addressRenderer = $addressRenderer;
        $this->logger = $logger;
        $this->date = $date;
        $this->appState = $appState;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return void
     */
    public function execute(Observer $observer)
    {
        /** @var OrderInterface $order */
        $order = $observer->getOrder();

        if (!$this->canSendEmail($order)) {
            return;
        }

        try {
            // Send mail to store
            $this->sendMail($order);
        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        }
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface | \Magento\Sales\Model\Order $order
     * @return bool
     */
    public function canSendEmail($order): bool
    {
        if (!$this->helper->getConfig('carriers/pickupinstore/active')) {
            return false;
        }

        $selectedOrderStatus = $this->helper->getConfig('carriers/pickupinstore/sender_email_order_status');

        if ($order->getStatus() != $selectedOrderStatus) {
            return false;
        }

        if (!$order->getPickupInStoreEmail()) {
            return false;
        }

        if (!$this->isOrderedWithPickUpInStore($order)) {
            return false;
        }

        if (!$order->getShippingAddress()) {
            return false;
        }

        if ($order->getShippingMethod() != 'pickupinstore_pickupinstore') {
            return false;
        }

        if ($order->getPickupInStoreEmailSendDate())  {
            return false;
        }

        return true;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderInterface $order
     * @return bool
     */
    protected function isOrderedWithPickUpInStore(OrderInterface $order)
    {
        if (!$order->getData('pickup_in_store_id')) {
            return false;
        }

        return true;
    }

    public function sendMail($order) {
        if ($this->appState->getAreaCode() === 'adminhtml') {
            $this->appState->emulateAreaCode(
                Area::AREA_FRONTEND,
                [$this, '_sendMail'],
                [$order]
            );
        } else {
            $this->_sendMail($order);
        }

    }

    /**
     * @param $order
     * @return void
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function _sendMail($order)
    {
        if (!$this->helper->getConfig('carriers/pickupinstore/email_to_store_enabled')) {
            return;
        }

        $orderStoreId = $order->getStoreId();

        $this->inlineTranslation->suspend();

        $billingAddress = $order->getBillingAddress();

        $templateVars = [
            'order'                    => $order,
            'billing'                  => $billingAddress,
            'payment_html'             => $this->getPaymentHtml($order),
            'store'                    => $orderStoreId,
            'formattedShippingAddress' => $this->getFormattedShippingAddress($order),
            'formattedBillingAddress'  => $this->getFormattedBillingAddress($order),
        ];

        $templateOptions = [
            'area'  => Area::AREA_FRONTEND,
            'store' => $orderStoreId,
        ];

        try {
            $this->transportBuilder
                ->setTemplateIdentifier($this->helper->getConfig('carriers/pickupinstore/email_to_store_template'))
                ->setTemplateOptions($templateOptions)
                ->setTemplateVars($templateVars)
                ->setFrom($this->helper->getConfig('carriers/pickupinstore/sender_email_to_store_identity'))
                ->addTo($order->getPickupInStoreEmail());

            $bcc = $this->helper->getConfig('carriers/pickupinstore/email_to_store_bcc_email');
            if (is_string($bcc) && $bcc != '') {
                $this->transportBuilder->addBcc(explode(',', $bcc));
            }

            $transport = $this->transportBuilder->getTransport();
            $transport->sendMessage();

            $this->inlineTranslation->resume();

            //Save Email send Date
            $order->setPickupInStoreEmailSendDate($this->date->gmtDate());
            $this->orderRepository->save($order);
        } catch (Exception $exception) {
            $this->inlineTranslation->resume();
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        }
    }

    /**
     * @param $order
     * @return string
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    protected function getPaymentHtml($order)
    {
        return $this->paymentHelper->getInfoBlockHtml(
            $order->getPayment(),
            $this->storeManager->getStore()->getId() != 0 ? $this->storeManager->getStore()->getId() : $order->getStoreId() //$this->storeManager->getStore()->getId()
        );
    }

    /**
     * @param $order
     * @return string|null
     */
    protected function getFormattedShippingAddress($order)
    {
        return $order->getIsVirtual()
            ? null
            : $this->addressRenderer->format($order->getShippingAddress(), 'html');
    }

    /**
     * @param $order
     * @return string|null
     */
    protected function getFormattedBillingAddress($order)
    {
        return $this->addressRenderer->format($order->getBillingAddress(), 'html');
    }
}
